@@ -313,11 +313,28 @@ def make_filename_safe(unistr, encoding=fs_enc, subst="_"):
 	
 	"""
 	# Turn characters that are invalid in the filesystem encoding into ASCII
-	# equivalents
-	unistr = unistr.encode(encoding, "safe_asciize").decode(encoding)
+	# substitution character '?'
+	# NOTE that under Windows, encoding with the filesystem encoding may
+	# substitute some characters even in "strict" replacement mode depending
+	# on the Windows language setting for non-Unicode programs! (Python 2.x
+	# under Windows supports Unicode by wrapping the win32 ASCII API, so it is
+	# a non-Unicode program from that point of view. This problem presumably
+	# doesn't exist with Python 3.x which uses the win32 Unicode API)
+	unidec = unistr.encode(encoding, "replace").decode(encoding)
+	# Replace substitution character '?' with ASCII equivalent of original char
+	uniout = u""
+	for i, c in enumerate(unidec):
+		if c == u"?":
+			# Note: We prevent IndexError by using slice notation which will
+			# return an empty string if unistr should be somehow shorter than
+			# unidec. Technically, this should never happen, but who knows
+			# what hidden bugs and quirks may linger in the Python 2.x Unicode
+			# implementation...
+			c = safe_asciize(unistr[i:i + 1])
+		uniout += c
 	# Remove invalid chars
-	unistr = re.sub(r"[\\/:*?\"<>|]+", subst, unistr)
-	return unistr
+	uniout = re.sub(r"[\\/:*?\"<>|]+", subst, uniout)
+	return uniout
 
 
 def normalencode(unistr, form="NFKD", encoding="ASCII", errors="ignore"):
@@ -352,8 +369,7 @@ def center(text, width = None):
 def create_replace_function(template, values):
 	""" Create a replace function for use with e.g. re.sub """
 	def replace_function(match, template=template, values=values):
-		for i, group in enumerate(match.groups()):
-			template = template.replace("\\%i" % (i + 1), group)
+		template = match.expand(template)
 		return template % values
 	return replace_function
 
@@ -457,7 +473,7 @@ def safe_basestring(obj):
 				obj = str(obj)
 			except UnicodeEncodeError:
 				obj = repr(obj)
-	if isinstance(oobj, Exception):
+	if isinstance(oobj, Exception) and not isinstance(oobj, Warning):
 		if obj and oobj.__class__.__name__ in dir(exceptions):
 			obj = obj[0].capitalize() + obj[1:]
 		module = getattr(oobj, "__module__", "")
